#!/usr/bin/env python
from bokeh.plotting import figure, output_file, show
from cbd.src.CBD import *
from cbd.src.CBDDraw import draw

"""
The following three classes declare the CBDs that are used here:
- Counter: a cbd that uses a delay block to generate the natural number sequence.
- Double: a cbd that doubles the value of its input
- EvenNumberGenerator: the cbd that puts the previous two together.
"""
class Counter(CBD):
    def __init__(self, block_name):
        CBD.__init__(self, 
                    block_name, 
                    input_ports=[], 
                    output_ports=["OutCount"])
        
        self.addBlock(DelayBlock(block_name="delay"))
        self.addBlock(AdderBlock(block_name="sum"))
        self.addBlock(ConstantBlock(block_name="zero", value=0.0))
        self.addBlock(ConstantBlock(block_name="one", value=1.0))
        
        self.addConnection("zero", "delay", 
                            input_port_name="IC")
        self.addConnection("delay", "OutCount")
        self.addConnection("delay", "sum")
        self.addConnection("sum", "delay", input_port_name="IN1")
        self.addConnection("one", "sum")
        
class Double(CBD):        
    def __init__(self, block_name):
        CBD.__init__(self, 
                     block_name, 
                     input_ports=["InNumber"], 
                     output_ports=["OutDouble"])
        self.addBlock(ProductBlock(block_name="mult"))
        self.addBlock(ConstantBlock(block_name="two", value=2.0))
        
        self.addConnection("InNumber", "mult")
        self.addConnection("two", "mult")
        self.addConnection("mult", "OutDouble")
    
    
class EvenNumberGen(CBD):
    def __init__(self, block_name):
        CBD.__init__(self, 
                     block_name, 
                     input_ports=[],
                     output_ports=["OutEven"])
        
        self.addBlock(Counter(block_name="counter"))
        self.addBlock(Double(block_name="double"))
        
        self.addConnection("counter", "double", 
                           input_port_name="InNumber",
                           output_port_name="OutCount")
        self.addConnection("double", "OutEven",
                           output_port_name="OutDouble")

"""
The following code shows how to run, render, and plot the results of, a cbd.
"""

# Create instance of EvenNumberGen CBD
cbd = EvenNumberGen("number_gen")

# Render EvenNumberGen cbd
cbdDot = draw(cbd)
cbdDot.render(filename='evenCBD', format='pdf', cleanup='true', view='true')
# Render Counter cbd
counterDot = draw(cbd.getBlockByName("counter"))
counterDot.render(filename='counterCBD', format='pdf', cleanup='true', view='true')

# Render Double cbd
counterDot = draw(cbd.getBlockByName("double"))
counterDot.render(filename='double', format='pdf', cleanup='true', view='true')

# Run EvenNumberGen CBD for 10 steps
cbd.run(10)

# Collects results and plot
times = []
output = []

for timeValuePair in cbd.getSignal("OutEven"):
    times.append(timeValuePair.time)
    output.append(timeValuePair.value)
            
#Plot
output_file("./number_gen.html", title="Even Numbers")
p = figure(title="Even Numbers", x_axis_label='time', y_axis_label='N')
p.circle(x=times, y=output, legend="Even numbers")
show(p)

